#!/bin/bash

module purge
module load CrayEnv
module load PrgEnv-cray/8.3.3
module load craype-accel-amd-gfx90a
module load rocm/5.2.3.lua

source /flash/project_462000125/samantao/lumi-training/miniconda3/bin/activate mlperf
LD_LIBRARY_PATH=$CONDA_PREFIX/lib:$LD_LIBRARY_PATH 

export NCCL_SOCKET_IFNAME=hsn0,hsn1,hsn2,hsn3


pkill python3

## DL params
export MAX_TOKENS=10240 
export LEARNING_RATE="1.976e-3" 
export WARMUP_UPDATES=1000
export EXTRA_PARAMS="--distributed-weight-update 0 --dwu-num-blocks 4 --dwu-num-rs-pg 2 --dwu-num-ar-pg 2 --dwu-num-ag-pg 0 --dwu-overlap-reductions --dwu-num-chunks 1 --dwu-flat-mt --dwu-compute-L2-grad-norm --max-source-positions 64 --max-target-positions 64 --adam-betas (0.9,0.98) "

## System run parms
export NNODES=1

## System config params
export NGPU=8
export SOCKETCORES=64 ##16 on MI100
export NSOCKET=1


#DATADIR="/tmp/sfantao/data-transformer/transformer-mlperf-data/v07-sumbission-data/wmt14_en_de/utf8"
LOGDIR="./results"

mkdir -p "${LOGDIR}"

SLURM_NTASKS_PER_NODE=${SLURM_NTASKS_PER_NODE:-$NGPU}


SEED=${SEED:-$RANDOM}
MAX_TOKENS=${MAX_TOKENS}
DATASET_DIR=${DATASET_DIR:-"/pfs/lustrep2/scratch/project_462000125/samantao/mlperf-data/data-transformer/transformer-mlperf-data/wmt14_en_de/utf8"}
NUMEPOCHS=${NUMEPOCHS:-1}

# Start timing
START=$(date +%s)
START_FMT=$(date +%Y-%m-%d\ %r)
echo "STARTING TIMING RUN AT ${START_FMT}"

if [[ ${WORLD_SIZE:-${NGPU}} -ne 1 ]]; then
    DISTRIBUTED_INIT_METHOD="--distributed-init-method env://"
else
    DISTRIBUTED_INIT_METHOD="--distributed-world-size 1"
fi

# These are scanned by train.py, so make sure they are exported
export DGXSYSTEM
export SLURM_NTASKS_PER_NODE
export SLURM_NNODES

declare -a CMD
if [ $NGPU == 1 ]; then
  CMD=('python')
else
  CMD=( 'python' '-u' '-m' 'bind_launch' "--nsockets_per_node=${NSOCKET}" \
    "--ncores_per_socket=${SOCKETCORES}" "--nproc_per_node=${NGPU}" )
fi

"${CMD[@]}" train.py ${DATASET_DIR} \
  --seed ${SEED} \
  --arch transformer_wmt_en_de_big_t2t \
  --share-all-embeddings \
  --optimizer adam \
  --adam-betas '(0.9, 0.997)' \
  --adam-eps "1e-9" \
  --clip-norm "0.0" \
  --lr-scheduler inverse_sqrt \
  --warmup-init-lr "0.0" \
  --warmup-updates ${WARMUP_UPDATES} \
  --lr ${LEARNING_RATE} \
  --min-lr "0.0" \
  --dropout "0.1" \
  --weight-decay "0.0" \
  --criterion label_smoothed_cross_entropy \
  --label-smoothing "0.1" \
  --max-tokens ${MAX_TOKENS} \
  --max-epoch ${NUMEPOCHS} \
  --target-bleu "25.0" \
  --ignore-case \
  --no-save \
  --update-freq 1 \
  --fp16 \
  --seq-len-multiple 2 \
  --source_lang en \
  --target_lang de \
  --bucket_growth_factor 1.035 \
  --batching_scheme "v0p5_better" \
  --batch_multiple_strategy "dynamic" \
  --fast-xentropy \
  --max-len-a 1 \
  --max-len-b 50 \
  --lenpen 0.6 \
  --no-progress-bar \
  --dataloader-num-workers 2 \
  --enable-dataloader-pin-memory \
  --multihead-attn-impl 'fast_with_lyrnrm_and_dropoutadd' \
  ${DISTRIBUTED_INIT_METHOD} \
  ${EXTRA_PARAMS} ; ret_code=$?

sleep 3
if [[ $ret_code != 0 ]]; then exit $ret_code; fi

# End timing
END=$(date +%s)
END_FMT=$(date +%Y-%m-%d\ %r)
echo "ENDING TIMING RUN AT ${END_FMT}"

# Report result
RESULT=$(( ${END} - ${START} ))
RESULT_NAME="transformer"
echo "RESULT,${RESULT_NAME},${SEED},${RESULT},${USER},${START_FMT}"
