#!/bin/bash

#
# Data
#
sc=/pfs/lustrep2/scratch/project_462000125/samantao/mlperf-data

#
# Load 
#
module purge
module load CrayEnv
module load PrgEnv-cray/8.3.3
module load craype-accel-amd-gfx90a
module load rocm/5.2.3.lua

source /flash/project_462000125/samantao/lumi-training/miniconda3/bin/activate mlperf
export LD_LIBRARY_PATH=$CONDA_PREFIX/lib:$LD_LIBRARY_PATH 


rm -rf run-trans.sh
cat > run-trans.sh << EOF
#!/bin/bash -ex

  if [ \$SLURM_LOCALID -eq 0 ] ; then
    #rm -rf /dev/shm/*
    rocm-smi || true
  else
    sleep 2
  fi
  
  echo "Rank \$SLURM_PROCID --> \$(taskset -p \$\$)"
  
  export NCCL_SOCKET_IFNAME=hsn0,hsn1,hsn2,hsn3
  
  case "\${SLURM_LOCALID}" in
  0)
    bcmd='numactl --physcpubind=50-55,114-119 --membind=3'
    ;;
  1)
    bcmd='numactl --physcpubind=58-63,122-127 --membind=3'
    ;;
  2)
    bcmd='numactl --physcpubind=18-23,82-87 --membind=1'
    ;;
  3)
    bcmd='numactl --physcpubind=26-31,90-95 --membind=1'
    ;;
  4)
    bcmd='numactl --physcpubind=2-7,66-71 --membind=0'
    ;;
  5)
    bcmd='numactl --physcpubind=10-15,74-79 --membind=0'
    ;;
  6)
    bcmd='numactl --physcpubind=34-39,98-103 --membind=2'
    ;;
  7)
    bcmd='numactl --physcpubind=42-47,106-111 --membind=2'
    ;;
  *)
    echo ==============================================================
    echo "ERROR: Unknown local rank \$SLURM_LOCALID"
    echo ==============================================================
    exit 1
    ;;
  esac
  
  export MASTER_ADDR=\$(scontrol show hostname "\$SLURM_NODELIST" | head -n1)
  export MASTER_PORT=29500
  export WORLD_SIZE=\$SLURM_NPROCS
  export RANK=\$SLURM_PROCID
  
  echo "--> MASTER_ADDR: \$MASTER_ADDR"
  echo "--> MASTER_PORT: \$MASTER_PORT"
  echo "--> WORLD_SIZE: \$WORLD_SIZE"
  echo "--> RANK: \$RANK" 
  
  \$bcmd \
  python -u train.py \
    --local_rank=\$SLURM_LOCALID \
    --distributed-world-size \$WORLD_SIZE \
    --distributed-rank \$RANK \
    --device-id 0 \
    $sc/data-transformer/transformer-mlperf-data/wmt14_en_de/utf8 \
    --seed 2028 \
    --arch transformer_wmt_en_de_big_t2t \
    --share-all-embeddings \
    --optimizer adam \
    --adam-betas '(0.9, 0.997)' \
    --adam-eps 1e-9 \
    --clip-norm 0.0 \
    --lr-scheduler inverse_sqrt \
    --warmup-init-lr 0.0 \
    --warmup-updates 1000 \
    --lr 1.976e-3 \
    --min-lr 0.0 \
    --dropout 0.1 \
    --weight-decay 0.0 \
    --criterion label_smoothed_cross_entropy \
    --label-smoothing 0.1 \
    --max-tokens 10240 \
    --max-epoch 1 \
    --target-bleu 25.0 \
    --ignore-case \
    --no-save \
    --update-freq 1 \
    --fp16 \
    --seq-len-multiple 2 \
    --source_lang en \
    --target_lang de \
    --bucket_growth_factor 1.035 \
    --batching_scheme v0p5_better \
    --batch_multiple_strategy dynamic \
    --fast-xentropy \
    --max-len-a 1 \
    --max-len-b 50 \
    --lenpen 0.6 \
    --no-progress-bar \
    --dataloader-num-workers 2 \
    --enable-dataloader-pin-memory \
    --multihead-attn-impl fast_with_lyrnrm_and_dropoutadd \
    --distributed-init-method 'env://' \
    --distributed-weight-update 0 \
    --dwu-num-blocks 4 \
    --dwu-num-rs-pg 2 \
    --dwu-num-ar-pg 2 \
    --dwu-num-ag-pg 0 \
    --dwu-overlap-reductions \
    --dwu-num-chunks 1 \
    --dwu-flat-mt \
    --dwu-compute-L2-grad-norm \
    --max-source-positions 64 \
    --max-target-positions 64 \
    --adam-betas '(0.9,0.98)' |& tee trans-\$SLURM_PROCID.log
EOF
chmod +x run-trans.sh

Nodes=$1
MYJOBID=$(squeue --me | tail -n 1 | awk '{print $1;}')

srun \
    --jobid=${MYJOBID} \
    -N $Nodes \
    -n $(($Nodes*8)) \
    --cpu-bind=none \
    --gpus $(($Nodes*8)) \
    ./run-trans.sh |& tee trans-all-ranks-$Nodes.log
